#!/bin/sh -f
. /opt/CPshared/5.0/tmp/.CPprofile.sh
ECHO=/bin/echo
SLEEP=/bin/sleep
TEST=/bin/test
DATE=/bin/date
CHMOD=/bin/chmod
TIMEOUT=/bin/timeout
SOURCE=source
VSENV=vsenv
CP="/bin/cp -r"
LN="/bin/ln -f"
FW=$FWDIR/bin/fw
PGREP=/bin/pgrep
SRC_CONF_DIR=$FWDIR/conf
SRC_AMW_DIR=$FWDIR/amw
SRC_MDS_FWDIR=$MDS_FWDIR
LOG=/opt/CPInstLog/tp_conf_autoupdate.elg 


# exit code should be unique, so each script start with different number
TP_CONF_IS_NOT_RUNNING=65
AMW_FETCH_LOCAL_FAILED=66
FILE_NOT_FOUND=67

# this hook is running as tp_conf_post_action.sh and as tp_conf_pre_action_revert.sh actions as defined in cd_utils installationPlan.xml and ds.conf
PACKAGE="TPCONF"
SCRIPT="install"

$ECHO "start ${PACKAGE} - ${SCRIPT} (v3)" >> $LOG
$DATE >> $LOG

reload_file_types_magic() {
  if [ ! -f $FWDIR/conf/itp_file_types.json ]; then
    $ECHO "file not found "$FWDIR/conf/itp_file_types.json >> $LOG
    exit ${FILE_NOT_FOUND}
  fi
  $MDS_FWDIR/bin/file_types_to_magic -from $FWDIR/conf/itp_file_types.json -to $FWDIR/database/itp_file_types.magic -mode itp >> $LOG

  $MDS_FWDIR/bin/file_types_to_magic -from $FWDIR/conf/itp_file_types.json -to $FWDIR/database/itp_trad_file_types.magic -mode trad >> $LOG

  if [ ! -f $FWDIR/conf/trad_file_types.json ]; then
    $ECHO "file not found "$FWDIR/conf/trad_file_types.json >> $LOG
    exit ${FILE_NOT_FOUND}
  fi
  $MDS_FWDIR/bin/file_types_to_magic -from $FWDIR/conf/trad_file_types.json -to $FWDIR/database/trad_file_types.magic -mode trad >> $LOG
}


wait_for_ready() {
	t=$1
	rc=1
	while [ $t -gt 0 -a $rc -ne 0 ]
	do
		t=`expr $t - 1`
		sleep 3
		curl_cli http://127.0.0.1:12872/global_policy/is_ab_installed >> $LOG
		rc=$?
		echo "" >> $LOG
	done
	return $rc
}


restart_services() {

	mode=$1
	$ECHO "mode="$mode >> $LOG
	
	service_pid=`$CPDIR/bin/cpwd_admin getpid -name TP_CONF_SERVICE $CPWD_CTX_ARGS`
	if [ $service_pid -ne 0 ]; then
		$CPDIR/bin/cpwd_admin stop -name TP_CONF_SERVICE $CPWD_CTX_ARGS >> $LOG
	fi

	$CPDIR/bin/cpwd_admin start -name TP_CONF_SERVICE -path $MDS_FWDIR/bin/tp_conf_service $CPWD_CTX_ARGS -command "tp_conf_service --conf=tp_conf.json --log=info" >> $LOG

	new_service_pid=`$CPDIR/bin/cpwd_admin getpid -name TP_CONF_SERVICE $CPWD_CTX_ARGS`
	if [ $new_service_pid -eq 0 ]; then
		$ECHO "tp_conf_service isn't running" >> $LOG
		\ls -l $FWDIR/amw/tp_conf_current_policy >> $LOG
		$TIMEOUT 5 tp_conf_service --conf=tp_conf.json --log=info >> $LOG
		\tail $FWDIR/log/tp_conf.log >> $LOG

		$ECHO "2" > $FWDIR/amw/stat/stat.tp_conf_autoupdate
		$ECHO "tp_conf update installation failed" >> $FWDIR/amw/stat/stat.tp_conf_autoupdate
		$ECHO "tp_conf_service isn't running" >> $FWDIR/amw/stat/stat.tp_conf_autoupdate
		exit ${TP_CONF_IS_NOT_RUNNING}
	fi

	wait_for_ready 30
	$MDS_FWDIR/bin/curl_cli -s http://localhost:12872/info >> $LOG

	reload_file_types_magic
        
	md5sum $FWDIR/conf/tp_conf.json  >> $LOG
	md5sum $FWDIR/conf/itp_gw.json  >> $LOG
	
	STATE_DIR=$FWDIR/state/local/AMW
	# if [ x$mode = x"INFINITY" ]; then
	if [ -f $STATE_DIR/local.set ]; then
	local no_policy=$(fw stat | grep 'defaultfilter\|InitialPolicy')
	if [ "$no_policy" = "" ]; then
		wait_for_ready 90
		rc=$?
		if [ $rc -ne 0 ]; then
			echo "wait_for_ready failed. rc="$rc >> $LOG
		fi

		$FW amw fetch local 2>&1 >> $LOG
		res=$?
		$ECHO "amw fetch ended with res="$res  >> $LOG
		$DATE >> $LOG
		if [[ $res -gt 0 ]] && [[ $res -lt 64 ]]; then
			$ECHO "amw fetch local failed" >> $LOG
			\tail $FWDIR/log/tp_conf.log >> $LOG
			$SLEEP 1
			$ECHO "about to run again with debug. check amw_fetch.elg"  >> $LOG
			$CPDIR/bin/cpwd_admin stop -name TP_CONF_SERVICE $CPWD_CTX_ARGS >> $LOG
			$CPDIR/bin/cpwd_admin start -name TP_CONF_SERVICE -path $MDS_FWDIR/bin/tp_conf_service $CPWD_CTX_ARGS -command "tp_conf_service --conf=tp_conf.json --log=debug" >> $LOG
			wait_for_ready 90
			netstat -anp | grep tp_conf >> $LOG
			export TDERROR_ALL_TPUTILS=5
			export TDERROR_ALL_CURL=5
			export TDERROR_ALL_FDT=5
			export TDERROR_DBG_OPT=-host,-prog,-pid,-tid
			TDERROR_ALL_MALWARE=5 $FW -d amw fetch local >& $FWDIR/log/amw_fetch.elg
			\tail -100 $FWDIR/log/amw_fetch.elg >> $LOG
			\tail -100 $FWDIR/log/tp_conf.elg >> $LOG

			$ECHO "2" > $FWDIR/amw/stat/stat.tp_conf_autoupdate
			$ECHO "tp_conf update installation failed" >> $FWDIR/amw/stat/stat.tp_conf_autoupdate
			$ECHO "amw fetch local failed" >> $FWDIR/amw/stat/stat.tp_conf_autoupdate
			exit ${AMW_FETCH_LOCAL_FAILED}
		fi

		# AMW_INSTALLED broadcast done by fetch
		
		# bg load_sigs will occur via fetch->AMW_INSTALLED->fwd->updateMgmtPackage->perform_update_now=TRUE
		# $ECHO "executing load_sigs in background" >> $LOG
		# $FW load_sigs -e -b AMW &

	fi
	fi

	$DATE >> $LOG
}

copy_profiles_and_conf() {
	$ECHO "sync files from VS0 to $1" >> $LOG

	# soft-links already created in the original (not current) tp_conf_new_vs.sh
	# but lets run tp_conf_new_vs.sh just in case we just upgraded from <R80.40
	if [ -f $SRC_MDS_FWDIR/scripts/tp_conf_new_vs.sh ]; then
		 $SRC_MDS_FWDIR/scripts/tp_conf_new_vs.sh $SRC_MDS_FWDIR $FWDIR >> $LOG
	fi

	# soft-link to (profiles) amw/tp_infinity_policy/ created in 
	# vsx_create_all_vs_dirs_internal_script.bash
	
	if [ ! -f $FWDIR/amw/tp_conf_current_policy ]; then
		# shouldn't happen
		$ECHO "creating tp_conf_current_policy" >> $LOG
		\echo '{"PolicyType": "NONE"}' > $FWDIR/amw/tp_conf_current_policy
	fi
	
	# new file that might not exist in existing (< R81.10) VSs
	\ln -s $SRC_CONF_DIR/settings_policy_defaults.json $FWDIR/conf
}



arch=`uname -p`
$ECHO "arch="$arch >> $LOG
if [ "$arch" == "x86_64" ]; then
	kver=`uname -a | sed -r -n -e 's/.* ([3-7])\.[0-9].*/\1/p'`
	echo "kver="$kver >> $LOG
	# 3.10 (R81.20), 4.18 (R82), 5.x (R82x)
	# example: Linux yalmog_172.23.63.99 4.18.0-372.9.1cpx86_64 #1 SMP Mon Jul 29 21:18:29 IDT 2024 x86_64 x86_64 x86_64 GNU/Linux
	if [ "$kver" -lt 5 ]; then
		echo "linux50" >> $LOG
		mv $FWDIR/bin/tp_conf_service_50    $FWDIR/bin/tp_conf_service    >> $LOG 2>&1
		mv $FWDIR/lib/libtp_conf_license_50.so $FWDIR/lib/libtp_conf_license.so >> $LOG 2>&1
	else
		echo "linux80" >> $LOG
		mv $FWDIR/bin/tp_conf_service_80    $FWDIR/bin/tp_conf_service    >> $LOG 2>&1
		mv $FWDIR/lib/libtp_conf_license_80.so $FWDIR/lib/libtp_conf_license.so >> $LOG 2>&1
	fi
else
	echo "ERROR: arch not supported" >> $LOG
	exit ${FILE_NOT_FOUND}
fi

if [ $($CPDIR/bin/cpprod_util FwIsVSX) -eq 1 ] ; then
	$ECHO "FW is VSX" >> $LOG
	$SOURCE /etc/profile.d/vsenv.sh
	vsenv 0  >> $LOG

	for VS_ID in $(/usr/bin/netns list)
	do
		vsenv $VS_ID 2>1 1>/dev/null
		if [ $? -eq 0 ]; then
			$ECHO "VS ID - ${VS_ID}" >> $LOG
			mode=`$MDS_FWDIR/bin/curl_cli -s http://localhost:12872/mode`
			copy_profiles_and_conf "${VS_ID}"
			CPWD_CTX_ARGS="-ctx ${VS_ID} -env inherit"
			restart_services $mode
			# tp_conf_service should not run in VSwitch (IS_VSW)
			FwIsVSBridge=`$CPDIR/bin/cpprod_util FwIsVSBridge | tr -d '[:space:]'`
			FwIsVSJunction=`$CPDIR/bin/cpprod_util FwIsVSJunction | tr -d '[:space:]'`
			$ECHO "VS ID - ${VS_ID} FwIsVSBridge='$FwIsVSBridge' FwIsVSJunction='$FwIsVSJunction'" >> $LOG
			if [ "$VS_ID" != "0" ] && [ "$FwIsVSBridge" = "1" ] && [ "$FwIsVSJunction" = "1" ]; then
				$ECHO "VS ID - ${VS_ID} VSwitch - stop tp_conf_service" >> $LOG
				$CPDIR/bin/cpwd_admin stop -name TP_CONF_SERVICE $CPWD_CTX_ARGS >> $LOG
			fi
		fi
	done
	vsenv 0  >> $LOG
	\rm -f $FWDIR/amw/tp_conf_current_policy.new
else
	# non-VSX
	$ECHO "FW is not VSX" >> $LOG
	
	. /etc/profile.d/mdpsenv.sh
	if [ -f $MDPS_MGMT_PLANE_CONF_FILE ]; then
		dplane 1>/dev/null 2>&1
	fi
	
	mode=`$MDS_FWDIR/bin/curl_cli -s http://localhost:12872/mode`
	restart_services $mode
fi

$ECHO "0" > $FWDIR/amw/stat/stat.tp_conf_autoupdate

$DATE >> $LOG
$ECHO "finish ${PACKAGE}" >> $LOG
exit 0
